{ config, lib, pkgs, ... }:

with lib;

let
  cfg = config.programs.xonsh;

  toPyValue = x:
    if builtins.isBool x then
      if x then "True" else "False"
    else if builtins.isString x then
      ''"${x}"''
    else if builtins.isList x then
      "[${lib.concatStringsSep ", " (map toPyValue x)}]"
    else if builtins.isAttrs x then
      "{${
        concatStringsSep ", "
        (mapAttrsToList (k: v: "${toPyValue k}: ${toPyValue v}") x)
      }}"
    else
      ''"${toString v}"'';
  define = n: v: "${n} = ${toPyValue v}";
  defineAll = vars: lib.concatStringsSep "\n" (lib.mapAttrsToList define vars);
  setenv = n: v: "\$${n} = ${toPyValue v}";
  setenvAll = vars: lib.concatStringsSep "\n" (lib.mapAttrsToList setenv vars);
  setdict = n: k: v: "${n}[${toPyValue k}] = ${toPyValue v}";
  setdictAll = dict: vars:
    lib.concatStringsSep "\n" (lib.mapAttrsToList (setdict dict) vars);

  aliasesStr = setdictAll "aliases" cfg.shellAliases;

  envVarsStr = setenvAll cfg.sessionVariables;

  xontribsStr = concatStringsSep " " cfg.xontribs;

  xontribSysPathStr = concatStringsSep "\n"
    (mapAttrsToList (k: v: "'${v}/${v.pythonModule.sitePackages}',")
      cfg.extraXontribs);

  extraXontribsStr = concatStringsSep " " (attrNames cfg.extraXontribs);
in {
  options.programs.xonsh = {
    enable = mkEnableOption "Xonsh Python-powered shell";

    shellAliases = mkOption {
      default = { };
      example = {
        ll = "ls -l";
        ".." = "cd ..";
      };
      description = ''
        An attribute set that maps aliases (the top level attribute names in
        this option) to command strings or directly to build outputs.
      '';
      type = types.attrsOf types.str;
    };

    xontribs = mkOption {
      default = [ "bashisms" ];
      example = [ "bashisms" "coreutils" ];
      description = ''
        A list of built-in xontribs to enable. For non-builtin xontribs see
        extraXontribs.
      '';
      type = types.listOf types.str;
    };

    extraXontribs = mkOption {
      default = { };
      example = literalExpression ''
        {
          direnv = buildPythonPackage { ... };
        }
      '';
      description = ''
        An attribute set that maps xontrib names to Python package derivations.
      '';
      type = types.attrsOf types.package;
    };

    sessionVariables = mkOption {
      default = { };
      type = types.attrs;
      example = { MAILCHECK = 30; };
      description = "Environment variables that will be set for xonsh session.";
    };

    rcExtra = mkOption {
      default = "";
      type = types.lines;
      description =
        "Extra commands that should be added to <filename>.xonshrc</filename>.";
    };
  };

  config = mkIf cfg.enable {
    home = {
      packages = [ pkgs.xonsh ];

      file.".xonshrc".text = ''
        import sys

        sys.path.extend([
        ${xontribSysPathStr}
        ])
        xontrib load ${xontribsStr}
        xontrib load ${extraXontribsStr}

        ${aliasesStr}

        # Environment variables
        source-bash "${config.home.profileDirectory}/etc/profile.d/hm-session-vars.sh"
        ${envVarsStr}

        ${cfg.rcExtra}
      '';
    };

  };
}
